@@ -1,5 +1,17 @@
 <?php
 
+/**
+  * SquirrelMail Restrict Senders Plugin
+  * Copyright (c) 2004-2005 Paul Lesneiwski <pdontthink@angrynerds.com>
+  * Licensed under the GNU GPL. For full terms see the file COPYING.
+  *
+  * @package plugins
+  * @subpackage restrict_senders
+  *
+  */
+
+
+
 // include compatibility plugin
 //
 if (defined('SM_PATH'))
@@ -11,8 +23,11 @@ else if (file_exists('./plugins/compatibility/functions.php'))
 
 
 
-// display errors
-//
+/**
+  * Displays any errors that blocked email from
+  * sending back on top of compose screen
+  *
+  */
 function display_error_on_compose_screen_do()
 {
 
@@ -46,17 +61,61 @@ function display_error_on_compose_screen_do()
          // here...
          //
          include_once (SM_PATH . 'plugins/restrict_senders/data/config.php');
-         global $countMe, $restrictNumberOfRecipients;
+         global $countMe, $restrictNumberOfRecipients, $vlogin_restrictNumberOfRecipients;
+         if (!empty($vlogin_restrictNumberOfRecipients)) 
+            $restrictNumberOfRecipients = $vlogin_restrictNumberOfRecipients;
+
          if (!$countMe)
          {
             $countMe = 1;
-            plain_error_message( _("Too Many Outgoing Recipients.  Please Limit Number Of Recipients To ") . $restrictNumberOfRecipients . '.', $color);
+            bindtextdomain ('restrict_senders', SM_PATH . 'locale');
+            textdomain ('restrict_senders');
+            plain_error_message( sprintf(_("Too many outgoing recipients. Please limit number of recipients to %d."), $restrictNumberOfRecipients), $color);
+            bindtextdomain ('squirrelmail', SM_PATH . 'locale');
+            textdomain ('squirrelmail');
             return;
          }
 
       }
 
 
+
+      // error when too many emails sent in one day
+      //
+      global $restrict_senders_error_too_many_emails_today, $color;
+      compatibility_sqextractGlobalVar('restrict_senders_error_too_many_emails_today');
+      if ($restrict_senders_error_too_many_emails_today)
+      {
+
+         // TODO: not sure if this will work with all
+         // PHP versions, especially 4.3.x....
+         //
+         // Need to avoid endless loops (plain_error_message()
+         // ends up calling to displayPageHeader(), which is
+         // where the current hook call is located, so this 
+         // code gets called recursively if we don't stop it
+         // here...
+         //
+         include_once (SM_PATH . 'plugins/restrict_senders/data/config.php');
+         global $countMe, $restrictNumberOfEmailsPerDay, $vlogin_restrictNumberOfEmailsPerDay;
+         if (!empty($vlogin_restrictNumberOfEmailsPerDay)) 
+            $restrictNumberOfEmailsPerDay = $vlogin_restrictNumberOfEmailsPerDay;
+
+         if (!$countMe)
+         {
+            $countMe = 1;
+            bindtextdomain ('restrict_senders', SM_PATH . 'locale');
+            textdomain ('restrict_senders');
+            plain_error_message( sprintf(_("Daily e-mail limit has been reached. Please limit number of emails per day to %d."), $restrictNumberOfEmailsPerDay), $color);
+            bindtextdomain ('squirrelmail', SM_PATH . 'locale');
+            textdomain ('squirrelmail');
+            return;
+         }
+
+      }
+
+
+
       // error when all To: recipients are disallowed
       //
       global $restrict_senders_error_no_to_recipients, $color;
@@ -77,7 +136,11 @@ function display_error_on_compose_screen_do()
          if (!$countMe)
          {
             $countMe = 1;
+            bindtextdomain ('restrict_senders', SM_PATH . 'locale');
+            textdomain ('restrict_senders');
             plain_error_message( _("All recipients in the To: field are illegal destinations."), $color);
+            bindtextdomain ('squirrelmail', SM_PATH . 'locale');
+            textdomain ('squirrelmail');
             return;
          }
 
@@ -89,9 +152,12 @@ function display_error_on_compose_screen_do()
 
 
 
-// remove an array element (by numeric 
-// index) and renumber the array
-//
+/**
+  * Utility function that removes an array 
+  * element (by numeric index) and renumbers
+  * the array
+  *
+  */
 function unset_and_renumber_array($theArray, $index)
 {
 
@@ -108,19 +174,30 @@ function unset_and_renumber_array($theArray, $index)
 
 
 
-// parses a table that lists domains and/or users that are
-// to be restricted to send only to certain domains 
-//
+/**
+  * Checks send restrictions when sending an email.
+  *
+  */
 function check_restrict_senders_do(&$argv)
 {
 
    global $restrictDomains, $restrictNumberOfRecipients, 
-          $restrict_sender_rules, $username, $domain, $blockedAddresses;
+          $vlogin_restrictNumberOfRecipients, $restrictNumberOfEmailsPerDay,
+          $restrict_sender_rules, $username, $domain, $blockedAddresses,
+          $vlogin_restrictNumberOfEmailsPerDay, $username, $data_dir;
 
 
    include_once (SM_PATH . 'plugins/restrict_senders/data/config.php');
 
 
+   // vlogin overrides
+   //
+   if (!empty($vlogin_restrictNumberOfRecipients)) 
+      $restrictNumberOfRecipients = $vlogin_restrictNumberOfRecipients;
+   if (!empty($vlogin_restrictNumberOfEmailsPerDay)) 
+      $restrictNumberOfEmailsPerDay = $vlogin_restrictNumberOfEmailsPerDay;
+
+
    // grab TO, CC, and BCC fields from message
    //
    $message = &$argv[1];
@@ -173,6 +250,66 @@ function check_restrict_senders_do(&$argv)
 
 
 
+   // limit number of emails sent per day
+   //
+   if ($restrictNumberOfEmailsPerDay)
+   {
+
+      // get and parse email count and date
+      //
+      list($year, $yearDay) = explode(':', date('Y:z'));
+      $emailCount = getPref($data_dir, $username, 'todays_email_count', 0);
+      if (!empty($emailCount))
+      {
+         list($day, $emailCount) = explode(':', $emailCount);
+         list($prefYear, $prefYearDay) = explode('_', $day);
+         if ($prefYear == $year && $prefYearDay == $yearDay)
+            $emailCount++;
+         else
+            $emailCount = 1;
+      }
+      else
+         $emailCount = 1;
+      
+
+      // check for overage
+      //
+      if ($emailCount > $restrictNumberOfEmailsPerDay)
+      {
+
+         global $color, $send_to, $send_to_cc, $send_to_bcc, $subject, $body,
+                $variable_sent_folder;
+         sqgetGlobalVar('variable_sent_folder', $variable_sent_folder);
+
+         header('Location: ../src/compose.php?send_to=' . urlencode($send_to) 
+            . '&send_to_cc=' . urlencode($send_to_cc) . '&send_to_bcc=' 
+            . urlencode($send_to_bcc) . '&subject=' . urlencode($subject) 
+            . '&restrict_senders_error_too_many_emails_today=1&body=' . urlencode($body)
+            . rs_buildQueryString('mailprio') . rs_buildQueryString('delete_draft') 
+            . rs_buildQueryString('identity') 
+            // doesn't pull default so just forget it! . rs_buildQueryString('variable_sent_folder') 
+            . rs_buildQueryString('session') 
+            // makes query string too big . rs_buildQueryString('restoremessages') 
+            . rs_buildQueryString('request_mdn') . rs_buildQueryString('request_dr') 
+            . rs_buildQueryString('passed_id') . rs_buildQueryString('custom_from') 
+            . rs_buildQueryString('mailbox') . rs_buildQueryString('passed_ent_id') 
+                // SM 1.2.x : reply_id and forward_id
+            . rs_buildQueryString('startMessage') . rs_buildQueryString('reply_id') 
+            . rs_buildQueryString('forward_id'));
+
+         exit;
+
+      }
+
+
+      // otherwise, continue, incrementing counter
+      //
+      setPref($data_dir, $username, 'todays_email_count', $year . '_' . $yearDay . ':' . $emailCount);
+
+   }
+
+
+
    // limit send domains?
    //
    if (!$restrictDomains)
@@ -369,14 +506,25 @@ function check_restrict_senders_do(&$argv)
 
 
 
-// build up location string
-// note that it always prepends & and
-// thus won't work for the first item
-// in the query string
-//
-// the variable will be renamed in the
-// query string if $queryName is given
-//
+/**
+  * Utility function that builds up a location string.
+  *
+  * Note that it always prepends & and thus won't work
+  * for the first item in the query string.
+  *
+  * The variable will be renamed in the query string 
+  * if $queryName is given
+  *
+  * @param string $varName The name of the variable to push 
+  *                        into query string style
+  * @param string $queryName The name of the variable to use
+  *                          instead of the variable name from 
+  *                          which the value comes (optional;
+  *                          default not used - same as var name).
+  *
+  * @return string Variable formatted in query string style
+  *
+  */
 function rs_buildQueryString($varName, $queryName = '')
 {
    global $$varName;
@@ -389,8 +537,13 @@ function rs_buildQueryString($varName, $queryName = '')
 
 
 
-// returns sender rules for the current user
-//
+/**
+  * Utility function that parses the sender rules configuration
+  * file and returns sender rules for the current user
+  *
+  * @return array A list of any sender rules found for the current user.
+  *
+  */
 function parseSenderRules()
 {
 
@@ -469,9 +622,13 @@ function parseSenderRules()
          {
 
             global $color;
-            plain_error_message( '"<b>restrict_senders</b>" ' . _("plugin is not configured correctly.") . '<br />' . _("Error on data/send_restrictions.dat line number ") . $lineNo . '<br /><center>' . _("Message not sent") . '</center>', $color);
+            bindtextdomain ('restrict_senders', SM_PATH . 'locale');
+            textdomain ('restrict_senders');
+            plain_error_message(sprintf(_("\"%s\" plugin is not configured correctly."), '<strong>restrict_senders</strong>') . '<br />' . sprintf(_("Error on data/send_restrictions.dat line number %d."), $lineNo) . '<br /><center>' . _("Message not sent.") . '</center>', $color);
             exit;
 ////////////      plain_error_message(_("The") . ' "<b>restrict_send</b>" ' . _("plugin has not been set up correctly.") . '<br />' . _("actual error message here.....") . '<br />' . _("advice to read instructions here......"), $color);
+            bindtextdomain ('squirrelmail', SM_PATH . 'locale');
+            textdomain ('squirrelmail');
 
          }
 
@@ -488,9 +645,13 @@ function parseSenderRules()
    {
 
       global $color;
-      plain_error_message( '"<b>restrict_senders</b>" ' . _("plugin could not open its restriction table") . '<br /><center>' . _("Message not sent") . '</center>', $color);
+      bindtextdomain ('restrict_senders', SM_PATH . 'locale');
+      textdomain ('restrict_senders');
+      plain_error_message(sprintf(_("\"%s\" plugin could not open its restriction table."), '<strong>restrict_senders</strong>') . '<br /><center>' . _("Message not sent.") . '</center>', $color);
       exit;
 ////////////      plain_error_message(_("The") . ' "<b>restrict_send</b>" ' . _("plugin has not been set up correctly.") . '<br />' . _("actual error message here.....") . '<br />' . _("advice to read instructions here......"), $color);
+      bindtextdomain ('squirrelmail', SM_PATH . 'locale');
+      textdomain ('squirrelmail');
 
    }
 
@@ -498,9 +659,20 @@ function parseSenderRules()
 
 
 
-// figures out which addresses are OK, 
-// which are bad based on the provided rules
-//
+/**
+  * Utility function that looks at all email addresses and determines
+  * which ones are OK and which ones are bad, according to the rules
+  * provided.
+  *
+  * @param array $toDomains All domains for To: addresses.
+  * @param array $ccDomains All domains for Cc: addresses.
+  * @param array $bccDomains All domains for Bcc: addresses.
+  * @param array $restrict_sender_rules The sender rules.
+  *
+  * @return array A three-element list in this order: $toResults, 
+  *               $ccResults, $bccResults
+  *
+  */
 function filterAddresses($toDomains, $ccDomains, $bccDomains, $restrict_sender_rules)
 {
 
